@@ -3,16 +3,17 @@
 //! Print some information about a Blender scene file.
 //! ```shell
 //! $ ./target/release/blend_info -h
-//! blend_info 0.2.3
-//! Print some information about a Blender scene file.
+//! blend_info 0.2.5
+//! Print some information about a Blender scene file
 //!
 //! USAGE:
 //!     blend_info [FLAGS] [OPTIONS] <path>
 //!
 //! FLAGS:
-//!         --dna        Print information about Blender's DNA
-//!     -h, --help       Prints help information
-//!     -V, --version    Prints version information
+//!         --dna         Print information about DNA of Blender
+//!     -h, --help        Prints help information
+//!         --pointers    Print code (e.g. OB, CA, LA, MA, DATA) and pointers
+//!     -V, --version     Prints version information
 //!
 //! OPTIONS:
 //!     -n, --struct_name <struct-name>    Print information about a particular struct
@@ -104,22 +105,22 @@
 //! }; // 120
 //! ```
 //! ```shell
-//! $ ./target/release/blend_info -n Object.ID.name blend/factory_v279.blend
-//! Object.ID.name = "OBCamera"
-//! Object.ID.name = "OBCube"
-//! Object.ID.name = "OBLamp"
+//! $ ./target/release/blend_info -n Object.id.name blend/factory_v279.blend
+//! Object.id.name = "OBCamera"
+//! Object.id.name = "OBCube"
+//! Object.id.name = "OBLamp"
 //! ```
 //! ```shell
-//! $ ./target/release/blend_info -n Camera.ID.name blend/factory_v279.blend
-//! Camera.ID.name = "CACamera"
+//! $ ./target/release/blend_info -n Camera.id.name blend/factory_v279.blend
+//! Camera.id.name = "CACamera"
 //! ```
 //! ```shell
-//! $ ./target/release/blend_info -n Lamp.ID.name blend/factory_v279.blend
-//! Lamp.ID.name = "LALamp"
+//! $ ./target/release/blend_info -n Lamp.id.name blend/factory_v279.blend
+//! Lamp.id.name = "LALamp"
 //! ```
 //! ```shell
-//! $ ./target/release/blend_info -n Mesh.ID.name blend/factory_v279.blend
-//! Mesh.ID.name = "MECube"
+//! $ ./target/release/blend_info -n Mesh.id.name blend/factory_v279.blend
+//! Mesh.id.name = "MECube"
 //! ```
 //! ```shell
 //! $ ./target/release/blend_info -n Camera.lens blend/factory_v279.blend
@@ -434,7 +435,9 @@ pub fn calc_mem_tlen(member: &DnaStrMember, type_found: u16) -> u16 {
     let mut chars = member.mem_name.chars();
     if let Some(c) = chars.next() {
         if c == '*' {
+            // pointers
             if let Some(cl) = chars.last() {
+                // array of pointers?
                 if cl == ']' {
                     // find number between square brackets ('some[number]')
                     let mut chars2 = member.mem_name.chars();
@@ -467,11 +470,25 @@ pub fn calc_mem_tlen(member: &DnaStrMember, type_found: u16) -> u16 {
                             * mem::size_of::<usize>() as u16;
                     }
                 } else {
+                    // simple pointer
                     mem_tlen = mem::size_of::<usize>() as u16;
                 }
             }
+        } else if c == '(' {
+            // functions?
+            if let Some(c2) = chars.next() {
+                if c2 == '*' {
+                    // function pointer
+                    mem_tlen = mem::size_of::<usize>() as u16;
+                } else {
+                    println!("TODO: {:?}", chars);
+                }
+            } else {
+                println!("TODO: {:?}", chars);
+            }
         } else {
             if let Some(cl) = chars.last() {
+                // arrays?
                 if cl == ']' {
                     // find number between square brackets ('some[number]')
                     let mut chars2 = member.mem_name.chars();
@@ -497,9 +514,11 @@ pub fn calc_mem_tlen(member: &DnaStrMember, type_found: u16) -> u16 {
                         mem_tlen = (numbers[0] as u16) * (numbers[1] as u16) * type_found;
                     }
                 } else {
+                    // simple type or struct
                     mem_tlen = type_found;
                 }
             } else {
+                // single letter name
                 mem_tlen = type_found;
             }
         }
@@ -540,6 +559,14 @@ pub fn get_id_name(
     return_str
 }
 
+pub fn get_char(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> u8 {
+    let mut char_value: u8 = 0;
+    if member.mem_type.as_str() == "char" {
+        char_value = bytes_read[byte_index];
+    }
+    char_value
+}
+
 pub fn get_float(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> f32 {
     let mut float_value: f32 = 0.0;
     if member.mem_type.as_str() == "float" {
@@ -599,6 +626,38 @@ pub fn get_int(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> i
     int_value
 }
 
+pub fn get_matrix(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> [f32; 16] {
+    let mut mat_values: [f32; 16] = [0.0_f32; 16];
+    if member.mem_type.as_str() == "float" {
+        let mut skip_bytes: usize = 0;
+        for i in 0..4 {
+            for j in 0..4 {
+                let mut mat_buf: [u8; 4] = [0_u8; 4];
+                for b in 0..4 as usize {
+                    mat_buf[b] = bytes_read[byte_index + skip_bytes + b];
+                }
+                let mat: f32 = unsafe { mem::transmute(mat_buf) };
+                mat_values[i * 4 + j] = mat;
+                skip_bytes += 4;
+            }
+        }
+    }
+    mat_values
+}
+
+pub fn get_pointer(_member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> usize {
+    let mut pointer_value: usize = 0;
+    pointer_value += (bytes_read[byte_index] as usize) << 0;
+    pointer_value += (bytes_read[byte_index + 1] as usize) << 8;
+    pointer_value += (bytes_read[byte_index + 2] as usize) << 16;
+    pointer_value += (bytes_read[byte_index + 3] as usize) << 24;
+    pointer_value += (bytes_read[byte_index + 4] as usize) << 32;
+    pointer_value += (bytes_read[byte_index + 5] as usize) << 40;
+    pointer_value += (bytes_read[byte_index + 6] as usize) << 48;
+    pointer_value += (bytes_read[byte_index + 7] as usize) << 56;
+    pointer_value
+}
+
 pub fn get_short(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> i16 {
     let mut short_value: i16 = 0;
     if member.mem_type.as_str() == "short" {
@@ -625,30 +684,30 @@ pub fn get_short3(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -
     short_values
 }
 
-pub fn get_matrix(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> [f32; 16] {
-    let mut mat_values: [f32; 16] = [0.0_f32; 16];
-    if member.mem_type.as_str() == "float" {
-        let mut skip_bytes: usize = 0;
-        for i in 0..4 {
-            for j in 0..4 {
-                let mut mat_buf: [u8; 4] = [0_u8; 4];
-                for b in 0..4 as usize {
-                    mat_buf[b] = bytes_read[byte_index + skip_bytes + b];
-                }
-                let mat: f32 = unsafe { mem::transmute(mat_buf) };
-                mat_values[i * 4 + j] = mat;
-                skip_bytes += 4;
-            }
+pub fn print_pointer(
+    pointer: usize,
+    struct_name: &String,
+    dna_pointers_hm: &HashMap<usize, usize>,
+) {
+    if pointer != 0_usize {
+        if let Some(pointer_found) = dna_pointers_hm.get(&pointer) {
+            println!(
+                "{} = {:#010x} ({:#018x})",
+                struct_name, pointer_found, pointer
+            );
         }
+    } else {
+        println!("{} = NULL", struct_name);
     }
-    mat_values
 }
 
 pub fn read_dna(
     print_dna: bool,
+    print_pointers: bool,
     path: &std::path::PathBuf,
     dna_types_hm: &mut HashMap<String, u16>,
     dna_structs_hm: &mut HashMap<String, DnaStrC>,
+    dna_pointers_hm: &mut HashMap<usize, usize>,
     dna_2_type_id: &mut Vec<u16>,
     types: &mut Vec<String>,
     bytes_read: &mut usize,
@@ -656,6 +715,7 @@ pub fn read_dna(
     let mut f = File::open(path)?;
     // read exactly 12 bytes
     let mut counter: usize = 0;
+    let mut chunk_start: usize;
     let mut buffer = [0; 12];
     f.read(&mut buffer)?;
     counter += 12;
@@ -668,6 +728,7 @@ pub fn read_dna(
         let mut current_code: String = String::new();
         let mut data_counter: u32 = 0;
         loop {
+            chunk_start = counter;
             // check 4 chars
             let mut buffer = [0; 4];
             f.read(&mut buffer)?;
@@ -704,10 +765,23 @@ pub fn read_dna(
                 // reset
                 data_counter = 0;
             }
-            // for now ignore the old entry
+            // use the old entry for pointer translation
             let mut buffer = [0; 8];
             f.read(&mut buffer)?;
             counter += 8;
+            let mut old: usize = 0;
+            old += (buffer[0] as usize) << 0;
+            old += (buffer[1] as usize) << 8;
+            old += (buffer[2] as usize) << 16;
+            old += (buffer[3] as usize) << 24;
+            old += (buffer[4] as usize) << 32;
+            old += (buffer[5] as usize) << 40;
+            old += (buffer[6] as usize) << 48;
+            old += (buffer[7] as usize) << 56;
+            if print_pointers {
+                println!("{:?} = {:#018x}", code, old);
+            }
+            dna_pointers_hm.insert(old, chunk_start);
             // get SDNAnr
             let mut buffer = [0; 4];
             f.read(&mut buffer)?;
@@ -1002,6 +1076,7 @@ pub fn use_dna(
     bytes_read: &mut Vec<u8>,
     structs_read: &mut Vec<String>,
     data_read: &mut Vec<u32>,
+    pointers_read: &mut Vec<(usize, u32)>,
 ) -> std::io::Result<()> {
     // find sdna_nr for each name entry
     let mut sdna_nrs: Vec<u32> = Vec::with_capacity(names.len());
@@ -1038,10 +1113,19 @@ pub fn use_dna(
             len += (buffer[1] as u32) << 8;
             len += (buffer[2] as u32) << 16;
             len += (buffer[3] as u32) << 24;
-            // for now ignore the old entry
+            // use the old entry for pointer translation
             let mut buffer = [0; 8];
             f.read(&mut buffer)?;
             counter += 8;
+            let mut old: usize = 0;
+            old += (buffer[0] as usize) << 0;
+            old += (buffer[1] as usize) << 8;
+            old += (buffer[2] as usize) << 16;
+            old += (buffer[3] as usize) << 24;
+            old += (buffer[4] as usize) << 32;
+            old += (buffer[5] as usize) << 40;
+            old += (buffer[6] as usize) << 48;
+            old += (buffer[7] as usize) << 56;
             // get SDNAnr
             let mut buffer = [0; 4];
             f.read(&mut buffer)?;
@@ -1089,6 +1173,7 @@ pub fn use_dna(
                         bytes_read.append(&mut buffer);
                         structs_read.push(name.clone());
                         data_read.push(len);
+			pointers_read.push((old, sdna_nr));
                     }
                 }
             } else {
@@ -1110,6 +1195,7 @@ pub fn use_dna(
                         bytes_read.append(&mut buffer);
                         structs_read.push(name.clone());
                         data_read.push(len);
+			pointers_read.push((old, sdna_nr));
                     }
                 }
             }
